/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core.config;

import java.awt.Dimension;
import java.awt.Point;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class WindowState {

    private Dimension size;
    private Point location;
    private Integer state;

    // \A\s*\[\s*(\d{1,4})\s*,\s*(\d{1,4})\s*,\s*(\d{1,4})\s*,\s*(\d{1,4})\s*,\s*([0-7])\s*\]\s*\z
    // matches a String like " [ 10, 20, 800, 600, 1 ] " corresponding to [ x, y, width, height, state ]
    private static final Pattern valuePattern = Pattern
            .compile("\\A\\s*\\[\\s*(\\d{1,4})\\s*,\\s*(\\d{1,4})\\s*,\\s*(\\d{1,4})\\s*,\\s*" +
                     "(\\d{1,4})\\s*,\\s*([0-7])\\s*\\]\\s*\\z");

    public WindowState(String value) throws IllegalArgumentException
    {
        if (value == null)
            throw new IllegalArgumentException("Paramter may not be null!");
        Matcher matcher = valuePattern.matcher(value);
        if (!matcher.matches())
            throw new IllegalArgumentException("String \"" + value
                    + "\" does not match [x, y, width, height, state]");

        location = new Point(Integer.parseInt(matcher.group(1)), Integer.parseInt(matcher.group(2)));
        size = new Dimension(Integer.parseInt(matcher.group(3)), Integer.parseInt(matcher.group(4)));
        state = Integer.valueOf(matcher.group(5));
    }

    public WindowState(Point location, Dimension size, int state) 
    {
        this.location = location;
        this.size = size;
        this.state = state;
    }

    public Dimension getSize()
    {
        return size;
    }

    public void setSize(Dimension size)
    {
        this.size = size;
    }

    public Point getLocation()
    {
        return location;
    }

    public void setLocation(Point location)
    {
        this.location = location;
    }

    public Integer getState()
    {
        return state;
    }

    public void setState(Integer state)
    {
        this.state = state;
    }

    @Override
    public String toString()
    {
        if (location == null || size == null || state == null)
            return null;

        return "[" + location.x + ", " + location.y + ", " + size.width + ", " + size.height + ", "
                + state + "]";
    }
}
