/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui;

import java.awt.Desktop;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;

import javax.swing.JTextPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;

import org.kuehntopf.streamtastic.StreamtasticApp;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * About Frame
 * @author  akuehntopf
 */
public class AboutFrame extends javax.swing.JFrame 
{
    
    /** 
     * Creates new form AboutFrame 
     */
    public 
    AboutFrame() 
    {
        initComponents();
        
        // Set Font
        String fontRule = "body {font-family: " + getFont().getFamily() + "; font-size: %s ;}";
        ((HTMLDocument) aboutTextPane.getDocument()).getStyleSheet().addRule(String.format(fontRule,getFont().getSize()));
        ((HTMLDocument) copyrightTextPane.getDocument()).getStyleSheet().addRule(String.format(fontRule,getFont().getSize()-1));

        // add hyperlink listener to open default browser
        HyperlinkListener hyperlinklistener = new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e)
            {
                if (e.getEventType() == HyperlinkEvent.EventType.ENTERED)
                    ((JTextPane)e.getSource()).setToolTipText(e.getURL().toString());
                else if (e.getEventType() == HyperlinkEvent.EventType.EXITED)
                    ((JTextPane)e.getSource()).setToolTipText(null);
                else if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED
                        && Desktop.isDesktopSupported())
                {
                    try
                    {
                        Desktop.getDesktop().browse(e.getURL().toURI());
                    }
                    catch (Exception ex)
                    {
                        System.err.println("Unable to open link in default browser!\n\tcause: "
                                + ex.getMessage());
                    }
                }
                // TODO handle case !Desktop.isDesktopSupported() (Linux/KDE3 affected)
                // possible solution: open message dialog and copy url to clipboard 
            }
        };
        aboutTextPane.addHyperlinkListener(hyperlinklistener);
        copyrightTextPane.addHyperlinkListener(hyperlinklistener);

        // ensure each pane is scrolled to top position
        copyrightTextPane.setCaretPosition(0);
        aboutTextPane.setCaretPosition(0);

        // dispose frame, if it is no longer focused
        addWindowFocusListener(new WindowFocusListener() {
            public void windowLostFocus(WindowEvent e)
            {
                dispose();
            }
            public void windowGainedFocus(WindowEvent e) {}
        });
    }
    
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        lblLogo = new javax.swing.JLabel();
        tabs = new javax.swing.JTabbedPane();
        scrollPaneAboutStreamtastic = new javax.swing.JScrollPane();
        aboutTextPane = new javax.swing.JTextPane();
        scrollPaneAboutCopyright = new javax.swing.JScrollPane();
        copyrightTextPane = new javax.swing.JTextPane();
        btnClose = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(Utilities.getString("ABOUT_DLG_TITLE")); // NOI18N
        setResizable(false);
        getContentPane().setLayout(new java.awt.GridBagLayout());

        lblLogo.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        lblLogo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/kuehntopf/streamtastic/resources/icons/streamtastic.png"))); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        getContentPane().add(lblLogo, gridBagConstraints);

        tabs.setBorder(new javax.swing.border.LineBorder(javax.swing.UIManager.getDefaults().getColor("InternalFrame.borderColor"), 1, true));
        tabs.setPreferredSize(new java.awt.Dimension(450, 200));

        scrollPaneAboutStreamtastic.setBorder(null);
        scrollPaneAboutStreamtastic.setOpaque(false);

        aboutTextPane.setBorder(null);
        aboutTextPane.setContentType("text/html");
        aboutTextPane.setEditable(false);
        aboutTextPane.setText(String.format(Utilities.getString("ABOUT_DLG_TAB_ABOUT_DESCRIPTION"), StreamtasticApp.getVersion())); // NOI18N
        aboutTextPane.setOpaque(false);
        scrollPaneAboutStreamtastic.setViewportView(aboutTextPane);

        tabs.addTab(Utilities.getString("ABOUT_DLG_TITLE"), scrollPaneAboutStreamtastic); // NOI18N

        scrollPaneAboutCopyright.setBorder(null);
        scrollPaneAboutCopyright.setOpaque(false);

        copyrightTextPane.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 10, 15, 10));
        copyrightTextPane.setContentType("text/html");
        copyrightTextPane.setEditable(false);
        copyrightTextPane.setText(Utilities.getString("ABOUT_DLG_TAB_COPYRIGHT_DESCRIPTION")); // NOI18N
        copyrightTextPane.setOpaque(false);
        scrollPaneAboutCopyright.setViewportView(copyrightTextPane);

        tabs.addTab(Utilities.getString("ABOUT_DLG_TAB_COPYRIGHT_TITLE"), scrollPaneAboutCopyright); // NOI18N

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 4);
        getContentPane().add(tabs, gridBagConstraints);

        btnClose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/kuehntopf/streamtastic/resources/icons/small/accept.png"))); // NOI18N
        btnClose.setText(Utilities.getString("CLOSE")); // NOI18N
        btnClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCloseActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        getContentPane().add(btnClose, gridBagConstraints);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCloseActionPerformed
        dispose();
    }//GEN-LAST:event_btnCloseActionPerformed
  
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextPane aboutTextPane;
    private javax.swing.JButton btnClose;
    private javax.swing.JTextPane copyrightTextPane;
    private javax.swing.JLabel lblLogo;
    private javax.swing.JScrollPane scrollPaneAboutCopyright;
    private javax.swing.JScrollPane scrollPaneAboutStreamtastic;
    private javax.swing.JTabbedPane tabs;
    // End of variables declaration//GEN-END:variables
    
}
