/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.LinearGradientPaint;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SpinnerModel;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.painter.MattePainter;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class ExpandableOptionPane extends JPanel implements ChangeListener {

    public static final int ALIGN_FLOW = 0;

    public static enum ParamType {
        CB_SIMPLE, CB_LONG_STRING, CB_SHORT_STRING, CB_SPINNER, LONG_STRING, CB_2SPINNER, 
        RB_SIMPLE, COMBO, TEXT_AREA
    }

    private static final String ENABLES_COMPONENTS = ExpandableOptionPane.class.getName()+".enables";

    private static final ImageIcon ICON_EXPANDED = Utilities.loadIcon("small/expanded.png");
    private static final ImageIcon ICON_COLLAPSED;
    static
    {
        BufferedImage icon = new BufferedImage(16, 16, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D g2d = (Graphics2D) icon.getGraphics();
        g2d.drawImage(ICON_EXPANDED.getImage(), AffineTransform.getRotateInstance(-Math.PI/2,8,8),null);
        ICON_COLLAPSED = new ImageIcon(icon);
    }

    private JPanel contentPane;
    private String title;
    private Map<Enum, JComponent> visuals = new HashMap<Enum, JComponent>();

    private JPanel currentGroup;
    private ButtonGroup currentButtonGroup, expandableOPaneGroup;
    private List<ButtonGroup> allButtonGroups = new ArrayList<ButtonGroup>();
    private final static GridBagConstraints CONSTRAINTS_GROUP, 
                                            CONSTRAINTS_SINGLE_COMP,
                                            CONSTRAINTS_SINGLE_SHIFTED_COMP, 
                                            CONSTRAINTS_LEFT_COMP, 
                                            CONSTRAINTS_RIGHT_HFILLED_COMP,
                                            CONSTRAINTS_RIGHT_COMP, 
                                            CONSTRAINTS_LTR_CONTAINER, 
                                            CONSTRAINTS_LTR_COMP;
    static
    { 
        // groups
        CONSTRAINTS_GROUP = new GridBagConstraints();
        CONSTRAINTS_GROUP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_GROUP.weightx = 1;
        CONSTRAINTS_GROUP.weighty = 0;
        CONSTRAINTS_GROUP.fill = GridBagConstraints.HORIZONTAL;
        CONSTRAINTS_GROUP.gridx = GridBagConstraints.REMAINDER;
        CONSTRAINTS_GROUP.insets = new Insets(5,5,10,5);

        // single component
        CONSTRAINTS_SINGLE_COMP = new GridBagConstraints();
        CONSTRAINTS_SINGLE_COMP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_SINGLE_COMP.weightx = 1;
        CONSTRAINTS_SINGLE_COMP.fill = GridBagConstraints.HORIZONTAL;
        CONSTRAINTS_SINGLE_COMP.gridwidth = GridBagConstraints.REMAINDER;
        CONSTRAINTS_SINGLE_COMP.insets = new Insets(0,10,0,10);

        // single shifted component
        CONSTRAINTS_SINGLE_SHIFTED_COMP = new GridBagConstraints();
        CONSTRAINTS_SINGLE_SHIFTED_COMP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_SINGLE_SHIFTED_COMP.weightx = 1;
        CONSTRAINTS_SINGLE_SHIFTED_COMP.fill = GridBagConstraints.HORIZONTAL;
        CONSTRAINTS_SINGLE_SHIFTED_COMP.gridwidth = GridBagConstraints.REMAINDER;
        CONSTRAINTS_SINGLE_SHIFTED_COMP.insets = new Insets(0,20,0,10);

        // 2 compoents
        CONSTRAINTS_LEFT_COMP = new GridBagConstraints();
        CONSTRAINTS_LEFT_COMP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_LEFT_COMP.insets = new Insets(0,10,0,5);

        CONSTRAINTS_RIGHT_HFILLED_COMP = new GridBagConstraints();
        CONSTRAINTS_RIGHT_HFILLED_COMP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_RIGHT_HFILLED_COMP.weightx = 1;
        CONSTRAINTS_RIGHT_HFILLED_COMP.fill = GridBagConstraints.HORIZONTAL;
        CONSTRAINTS_RIGHT_HFILLED_COMP.gridwidth = GridBagConstraints.REMAINDER;
        CONSTRAINTS_RIGHT_HFILLED_COMP.insets = new Insets(0,0,0,10);

        CONSTRAINTS_RIGHT_COMP = new GridBagConstraints();
        CONSTRAINTS_RIGHT_COMP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_RIGHT_COMP.weightx = 1;
        CONSTRAINTS_RIGHT_COMP.gridwidth = GridBagConstraints.REMAINDER;
        CONSTRAINTS_RIGHT_COMP.insets = new Insets(0,0,0,10);

        // left to right flow
        CONSTRAINTS_LTR_CONTAINER = new GridBagConstraints();
        CONSTRAINTS_LTR_CONTAINER.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_LTR_CONTAINER.weightx = 1;
        CONSTRAINTS_LTR_CONTAINER.gridwidth = GridBagConstraints.REMAINDER;
        CONSTRAINTS_LTR_CONTAINER.insets = new Insets(0,10,0,10);

        CONSTRAINTS_LTR_COMP = new GridBagConstraints();
        CONSTRAINTS_LTR_COMP.anchor = GridBagConstraints.WEST;
        CONSTRAINTS_LTR_COMP.insets = new Insets(0,0,0,2);
    }

    private AbstractButton selector;

    public ExpandableOptionPane(String title)
    {
        this(title, true, null);
    }

    public ExpandableOptionPane(String title, boolean selectable, ButtonGroup group)
    {
        super();
        this.expandableOPaneGroup = group;
        this.contentPane = new JPanel();
        this.title = title;
        setBorder(new LineBorder(new Color(0x60808080,true), 1, true));

        setLayout(new BorderLayout());
        initHeader(selectable, group);
        add(contentPane, BorderLayout.CENTER);

        contentPane.setVisible(false);
        contentPane.setLayout(new GridBagLayout());
    }

    private void initHeader(boolean selectable, ButtonGroup group)
    {
        JXPanel header = new JXPanel();
        header.setBackgroundPainter(new MattePainter(new LinearGradientPaint(0.0f, 0.0f, 800, 30,
                new float[] { 0.0f, 1.0f }, new Color[] { new Color(0x18808080 ,true), new Color(0x50808080 ,true) })));
        header.setLayout(new BorderLayout());

        if (selectable)
            selector = new JCheckBox(title);
        else
        {
            JRadioButton radiobutton = new JRadioButton(title);
            radiobutton.setIcon(ICON_COLLAPSED);
            radiobutton.setRolloverIcon(radiobutton.getIcon());
            radiobutton.setSelectedIcon(ICON_EXPANDED);
            radiobutton.setRolloverSelectedIcon(radiobutton.getSelectedIcon());
            if (group!=null) group.add(radiobutton);
            selector = radiobutton;
        }

        header.add(selector, BorderLayout.CENTER);
        add(header, BorderLayout.NORTH);
        selector.setContentAreaFilled(false);
        selector.addChangeListener(this);
    }

    public void setExpanded(boolean expanded)
    {
        if (!expanded && expandableOPaneGroup!=null && selector.isSelected()) expandableOPaneGroup.clearSelection();
        else selector.setSelected(expanded);
    }

    public boolean isExpanded()
    {
        return selector.isSelected();
    }

    public Enum[] getParams()
    {
        return visuals.keySet().toArray(new Enum[visuals.keySet().size()]);
    }

    public boolean isSet(Enum target)
    {
        JComponent comp = visuals.get(target);
        if (comp == null) return false;

        if (comp instanceof JCheckBox) return ((JCheckBox)comp).isSelected();
        if (comp instanceof StringValueEditor) return true;
        if (comp instanceof JRadioButton) return ((JRadioButton)comp).isSelected();

        throw new IllegalArgumentException("isSet is not defined for: "+target);
    }

    private void setSelectedForButtonGroup(JRadioButton button, boolean selected)
    {
        // setSelected(false) is not working, if the button is part of a buttongroup
        // in that case we need to invoke clearSelection() on the button group
        if (selected) button.setSelected(selected);
        else if (button.isSelected())
            for (ButtonGroup group: allButtonGroups)
                if (group.isSelected(button.getModel())) group.clearSelection();
    }

    public void setValue(Enum target, String value)
    {
        JComponent comp = visuals.get(target);
        if (comp==null) throw new IllegalArgumentException("No Parameter set for: "+target);

        // LONG_STRING:
        if (comp instanceof StringValueEditor) ((StringValueEditor) comp).getTextField().setText(value);
        // COMBO:
        if (comp instanceof JComboBox) {
            ComboBoxModel data = ((JComboBox) comp).getModel();
            for (int i = 0; i < data.getSize(); i++)
                if (data.getElementAt(i).equals(value)) {
                    data.setSelectedItem(data.getElementAt(i));
                    break;
                }
        }
        // TEXT_AREA:
        if (comp instanceof JTextPane) ((JTextPane) comp).setText(value);
        // RB_SIMPLE:
        if (comp instanceof JRadioButton) setSelectedForButtonGroup((JRadioButton) comp, value!=null);

        if (!(comp instanceof JCheckBox)) return;
        ((JCheckBox)comp).setSelected(value!=null);
        JComponent[] valueComps = (JComponent[]) comp.getClientProperty(ENABLES_COMPONENTS);

        // CB_SIMPLE:
        if (valueComps == null) ((JCheckBox) comp).setSelected(value!=null);
        // CB_2SPINNER:
        else if (valueComps.length == 2)
        {
            if (value == null) return;
            String[] values = value.split(":");
            if (values.length!=2) return;
            ((JSpinner)valueComps[0]).setValue(Integer.valueOf(values[0]));
            ((JSpinner)valueComps[1]).setValue(Integer.valueOf(values[1]));
        }
        // CB_LONG_STRING, CB_SHORT_STRING:
        else if ( valueComps[0] instanceof StringValueEditor && valueComps[0].isEnabled()) ((StringValueEditor)valueComps[0]).getTextField().setText(value);
        // CB_SPINNER:
        else if ( valueComps[0] instanceof JSpinner && value!=null && value.length()!=0 ) ((JSpinner)valueComps[0]).setValue(Integer.valueOf(value));
    }

    public String getValue(Enum target)
    {
        JComponent comp = visuals.get(target);
        if (comp==null) throw new IllegalArgumentException("No Parameter set for: "+target);

        // LONG_STRING:
        if (comp instanceof StringValueEditor) return ((StringValueEditor) comp).getTextField().getText().trim();
        // COMBO:
        if (comp instanceof JComboBox)
            return ((ComboBoxEntry)((JComboBox) comp).getSelectedItem()).getValue().toString();
        // TEXT_AREA:
        if (comp instanceof JTextPane) return ((JTextPane) comp).getText().trim();
        // RB_SIMPLE:
        if (comp instanceof JRadioButton) return target.toString();

        JComponent[] valueComps = (JComponent[]) comp.getClientProperty(ENABLES_COMPONENTS);
        // CB_SIMPLE:
        if (valueComps == null) return null;

        // return null in case that checkbox is not selected 
        if (!isSet(target)) return null;

        // CB_2SPINNER:
        if (valueComps.length == 2) return ((JSpinner)valueComps[0]).getValue()+":"+((JSpinner)valueComps[1]).getValue();
        // CB_LONG_STRING, CB_SHORT_STRING:
        if ( valueComps[0] instanceof StringValueEditor) return ((StringValueEditor)valueComps[0]).getTextField().getText().trim();
        // CB_SPINNER:
        if ( valueComps[0] instanceof JSpinner) return ((JSpinner)valueComps[0]).getValue().toString();

        throw new IllegalArgumentException("getValue() not defined for: "+target);
    }

    public void startGroup(String title)
    {
        currentButtonGroup = null;
        currentGroup = new JPanel();
        currentGroup.setBorder(new TitledBorder(title));
        currentGroup.setLayout(new GridBagLayout());
        contentPane.add(currentGroup, CONSTRAINTS_GROUP);
    }

    public void addParam(String label, Enum target, ParamType type)
    {
        switch (type)
        {
            case CB_LONG_STRING:
            case LONG_STRING: addParam(label, target, type, 1); break;
            default: addParam(label, target, type, ALIGN_FLOW);
        }
    }

    public void addParam(String label, Enum target, ParamType type, int align)
    {
        switch (type) 
        {
            case CB_SIMPLE: createCheckBox(label, target); break;
            case CB_LONG_STRING: createCheckBoxWithLargeTextfield(label, target, align); break;
            case CB_SHORT_STRING: createCheckBoxWithSmallTextField(label, target, align); break;
            case CB_SPINNER: createCheckBoxWithSpinner(label, target, align); break;
            case LONG_STRING: createLabelWithLargeTextfield(label, target, align); break;
            case COMBO: createLabelWithComboBox(label, target, align); break;
            case TEXT_AREA: createLabelWithTextArea(label, target); break;
            case CB_2SPINNER: createCheckBoxWith2Spinner(label, target, align); break;
            case RB_SIMPLE: createRadioButton(label, target); break;
        }
    }

    public void setValueSetForCombo(Enum target, String[] labels, Object[] values)
    {
        Component c = visuals.get(target);
        if (!(c instanceof JComboBox))
            throw new IllegalArgumentException(target + "is not of type: " + ParamType.COMBO);

        JComboBox combo = (JComboBox) c;
        for (int i=0; i < Math.min(labels.length, values.length); i++)
            combo.addItem(new ComboBoxEntry(labels[i], values[i]));
    }

    public void setCustomEditorForTextField(Enum target, CustomEditor editor)
    {
        JComponent c = visuals.get(target);
        if (c instanceof JCheckBox && c.getClientProperty(ENABLES_COMPONENTS) != null)
            c = ((JComponent[]) c.getClientProperty(ENABLES_COMPONENTS))[0];
        if (!(c instanceof StringValueEditor))
            throw new IllegalArgumentException(target + "is not of type: {" + ParamType.CB_LONG_STRING
                    + ", " + ParamType.CB_SHORT_STRING + ", " + ParamType.LONG_STRING + "}");

        StringValueEditor text = (StringValueEditor) c;
        text.setCustomEditor(editor);
    }

    public void setModelForSpinner(Enum target, SpinnerModel... models) 
    {
        JComponent c = visuals.get(target);
        if (c instanceof JCheckBox && c.getClientProperty(ENABLES_COMPONENTS) != null)
        {
            JComponent[] cs = (JComponent[]) c.getClientProperty(ENABLES_COMPONENTS);
            for (int i=0; i < Math.min(models.length, cs.length); i++) 
                if (cs[i] instanceof JSpinner)
                {
                    ((JSpinner) cs[i]).setModel(models[i]);
                    ((JSpinner.DefaultEditor)((JSpinner) cs[i]).getEditor()).getTextField().setColumns(5);
                }
            return;
        }
        throw new IllegalArgumentException(target + "is not of type: {" + ParamType.CB_2SPINNER
                + ", " + ParamType.CB_SPINNER + "}");
    }

    public void setTooltipForComponents(Enum target, String... tooltips)
    {
        JComponent c = visuals.get(target);
        if (tooltips.length>=1) c.setToolTipText(tooltips[0]);
        if (c.getClientProperty(ENABLES_COMPONENTS) != null)
        {
            JComponent[] cs = (JComponent[]) c.getClientProperty(ENABLES_COMPONENTS);
            for (int i=0; i < Math.min(tooltips.length-1, cs.length); i++) 
                    cs[i].setToolTipText(tooltips[i+1]);
        }
    }

    private void createLabelWithTextArea(String label, Enum target)
    {
        JLabel lbl = new JLabel(label);
        (currentGroup==null?contentPane:currentGroup).add(lbl, CONSTRAINTS_SINGLE_COMP);

        final JTextPane text = new JTextPane();
        text.setBorder(null); // remove double border in substance
        JScrollPane scroller = new JScrollPane(text);
        scroller.setPreferredSize(new Dimension(100, text.getPreferredSize().height * 4));

        visuals.put(target, text);
        (currentGroup==null?contentPane:currentGroup).add(scroller, CONSTRAINTS_SINGLE_SHIFTED_COMP);
    }

    private void createLabelWithComboBox(String label, Enum target, int align)
    {
        JComboBox combo = new JComboBox();

        visuals.put(target, combo);
        layout(align, false, new JLabel(label), combo);
    }

    private void createRadioButton(String label, Enum target)
    {
        if (currentButtonGroup==null)
        {
            currentButtonGroup = new ButtonGroup();
            allButtonGroups.add(currentButtonGroup);
        }

        JRadioButton rbut = new JRadioButton(label);
        currentButtonGroup.add(rbut);

        visuals.put(target, rbut);
        (currentGroup==null?contentPane:currentGroup).add(rbut, CONSTRAINTS_SINGLE_COMP);
    }

    private void createCheckBoxWith2Spinner(String label, Enum target, int align)
    {
        JCheckBox cbox = new JCheckBox(label);

        JSpinner spinner = new JSpinner();
        spinner.setEnabled(cbox.isSelected());
        ((JSpinner.DefaultEditor)spinner.getEditor()).getTextField().setColumns(5);

        JSpinner spinner2 = new JSpinner();
        spinner2.setEnabled(cbox.isSelected());
        ((JSpinner.DefaultEditor)spinner2.getEditor()).getTextField().setColumns(5);

        cbox.putClientProperty(ENABLES_COMPONENTS, new JComponent[]{spinner, spinner2});
        cbox.addChangeListener(this);

        visuals.put(target, cbox);
        layout(align, false, cbox, spinner, new JLabel(":"), spinner2);
    }

    private void createLabelWithLargeTextfield(String label, Enum target, int align)
    {
        JLabel lbl = new JLabel(label);
        StringValueEditor text = new StringValueEditor();

        visuals.put(target, text);
        layout(align, true, lbl, text);
    }

    private void createCheckBox(String label, Enum target)
    {
        JCheckBox cbox = new JCheckBox(label);

        visuals.put(target, cbox);
        (currentGroup==null?contentPane:currentGroup).add(cbox, CONSTRAINTS_SINGLE_COMP);
    }

    private void createCheckBoxWithLargeTextfield(String label, Enum target, int align)
    {
        JCheckBox cbox = new JCheckBox(label);

        StringValueEditor text = new StringValueEditor();
        text.setEnabled(cbox.isSelected());

        cbox.putClientProperty(ENABLES_COMPONENTS, new JComponent[]{text});
        cbox.addChangeListener(this);

        visuals.put(target, cbox);
        layout(align, true, cbox, text);
    }

    private void createCheckBoxWithSpinner(String label, Enum target, int align)
    {
        JCheckBox cbox = new JCheckBox(label);

        JSpinner spinner = new JSpinner();
        spinner.setEnabled(cbox.isSelected());
        ((JSpinner.DefaultEditor)spinner.getEditor()).getTextField().setColumns(5);

        cbox.putClientProperty(ENABLES_COMPONENTS, new JComponent[]{spinner});
        cbox.addChangeListener(this);

        visuals.put(target, cbox);
        layout(align, false, cbox, spinner);
    }

    private void createCheckBoxWithSmallTextField(String label, Enum target, int align)
    {
        JCheckBox cbox = new JCheckBox(label);

        StringValueEditor text = new StringValueEditor();
        text.setEnabled(cbox.isSelected());

        cbox.putClientProperty(ENABLES_COMPONENTS, new JComponent[]{text});
        cbox.addChangeListener(this);

        visuals.put(target, cbox);
        layout(align, false, cbox, text);
    }

    private void layout(int align, boolean hfill, JComponent... components)
    {
        if (align <= ALIGN_FLOW)
        {
            JPanel hgroup = new JPanel();
            hgroup.setLayout(new GridBagLayout());
            (currentGroup==null?contentPane:currentGroup).add(hgroup, CONSTRAINTS_LTR_CONTAINER);

            for (JComponent comp: components) hgroup.add(comp, CONSTRAINTS_LTR_COMP);
        }
        else 
        {
            GridBagConstraints constraints = CONSTRAINTS_LEFT_COMP;
            if (align > 1)
            {
                constraints = (GridBagConstraints) CONSTRAINTS_LEFT_COMP.clone();
                constraints.gridwidth = align;
            }
            (currentGroup==null?contentPane:currentGroup).add(components[0], constraints);

            constraints = hfill ? CONSTRAINTS_RIGHT_HFILLED_COMP : CONSTRAINTS_RIGHT_COMP;
            if (components.length == 2)
                (currentGroup==null?contentPane:currentGroup).add(components[1], constraints);
            else 
            {
                JPanel hgroup = new JPanel();
                hgroup.setLayout(new GridBagLayout());

                for (int i=1; i < components.length; i++) hgroup.add(components[i], CONSTRAINTS_LTR_COMP);
                (currentGroup==null?contentPane:currentGroup).add(hgroup, constraints);
            }
        }
    }

    public void stateChanged(ChangeEvent e)
    {
        if (e.getSource() == selector)
        {
            contentPane.setVisible(selector.isSelected());
            return;
        }

        if (e.getSource() instanceof JCheckBox)
        {
            JCheckBox src = (JCheckBox) e.getSource();
            JComponent[] enables = (JComponent[]) src.getClientProperty(ENABLES_COMPONENTS);
            if (enables!= null) for (JComponent comp : enables )
                comp.setEnabled(src.isSelected());
            return;
        }
    }
}

class ComboBoxEntry {
    private String label;
    private Object value;

    public ComboBoxEntry(String label, Object value)
    {
        super();
        this.label = label;
        this.value = value;
    }
    @Override
    public int hashCode()
    {
        return value.hashCode();
    }
    @Override
    public boolean equals(Object obj)
    {
        if (this == obj || value.equals(obj)) return true;
        if (getClass() != obj.getClass()) return false;
        final ComboBoxEntry other = (ComboBoxEntry) obj;
        if (value == null)
        {
            if (other.value != null) return false;
        }
        else if (!value.equals(other.value)) return false;

        return true;
    }
    @Override
    public String toString()
    {
        return label;
    }
    public Object getValue()
    {
        return value;
    }
}

class StringValueEditor extends JPanel implements ActionListener {

    private JTextField text = new JTextField();
    private JButton button = new JButton("...");
    private CustomEditor editor;

    {
        setLayout(new BorderLayout());
        add(text, BorderLayout.CENTER);
        add(button, BorderLayout.EAST);
        button.setVisible(false);
        button.addActionListener(this);

        // layout options
        text.setColumns(12);
        setPreferredSize(text.getPreferredSize());
        button.setMinimumSize(new Dimension(button.getMinimumSize().height,button.getMinimumSize().height));
        button.setPreferredSize(button.getMinimumSize());
    }

    public JTextField getTextField() {
        return text;
    }

    @Override
    public void setToolTipText(String tooltip)
    {
        text.setToolTipText(tooltip);
    }

    public void setCustomEditor(CustomEditor editor) {
        this.editor = editor;
        button.setVisible(editor!=null);
    }

    public void actionPerformed(ActionEvent e)
    {
        if (editor!=null) text.setText(editor.edit(this, text.getText()));
    }

    @Override
    public void setEnabled(boolean enabled)
    {
        super.setEnabled(enabled);
        text.setEnabled(enabled);
        button.setEnabled(enabled);
    }
}