/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.models;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.core.config.ParameterInfo;
import org.kuehntopf.streamtastic.core.config.Config.Key;
import org.kuehntopf.streamtastic.core.streamripper.StreamripperCommand;
import org.kuehntopf.streamtastic.utils.Player;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class AdvancedPreferencesModel extends AbstractTableModel implements PropertyChangeListener {

    private List<Parameter> parameters = new ArrayList<Parameter>();

    /**
     * 
     */
    public AdvancedPreferencesModel()
    {
        Config.getInstance().addPropertyChangeListener(this);

        for (Config.Key k : Config.Key.values())
            parameters.add(new Parameter(k));
    }

    public TableColumnModel getColumnModel() 
    {
        TableColumnModel model = new DefaultTableColumnModel();
        model.addColumn(new ParameterColumn(ParameterColumn.LABEL,50));
        model.addColumn(new ParameterColumn(ParameterColumn.VALUE,150));
        return model;
    }

    public void propertyChange(PropertyChangeEvent evt)
    {
        fireTableCellUpdated(Config.Key.valueOf(evt.getPropertyName()).ordinal(), 1);
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount()
    {
        return 2;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount()
    {
        return parameters.size();
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return columnIndex == 1 && !parameters.get(rowIndex).isReadonly();
    }

    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
        fireTableRowsUpdated(rowIndex, rowIndex);
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt(int rowIndex, int columnIndex)
    {
        return parameters.get(rowIndex);
    }

    public void applyAll()
    {
        for (Parameter p : parameters) p.apply();
        fireTableRowsUpdated(0, parameters.size()-1);
    }

    public void revertAll()
    {
        for (Parameter p : parameters) p.revert();
        fireTableRowsUpdated(0, parameters.size()-1);
    }

    public void restoreAll()
    {
        for (Parameter p : parameters) p.reset();
        fireTableRowsUpdated(0, parameters.size()-1);
    }

    public String getValue(Key key)
    {
        return getParameter(key).getValue();
    }

    public void setValue(Key key, String value)
    {
        getParameter(key).setValue(value);
        fireTableRowsUpdated(key.ordinal(), key.ordinal());
    }

    public void restore(Key key)
    {
        getParameter(key).reset();
        fireTableRowsUpdated(key.ordinal(), key.ordinal());
    }

    public void revert(Key key)
    {
        getParameter(key).revert();
        fireTableRowsUpdated(key.ordinal(), key.ordinal());
    }

    public void apply(Key key)
    {
        getParameter(key).apply();
        fireTableRowsUpdated(key.ordinal(), key.ordinal());
    }

    public boolean isModified(Key key)
    {
        return getParameter(key).isRevertable();
    }

    public boolean isModified()
    {
        for (Parameter parameter: parameters)
            if (parameter.isRevertable()) return true;
        return false;
    }

    private Parameter getParameter(Key key)
    {
        Parameter p = parameters.get(key.ordinal());
        if (p.getKey() != key)
            throw new IllegalStateException("parameters and keys are out of sync!!!");
        return p;
    }
}

class Parameter {
    private String newValue;
    private Config.Key target;

    protected Parameter(Config.Key key)
    {
        this.target = key;
    }

    public Config.Key getKey() {
        return target;
    }

    public String getLabel()
    {
        return target.toString();
    }

    public void setValue(String value)
    {
        String oldValue = Config.getInstance().getProperty(target);
        try
        {
            if (((oldValue == null && value != null) || oldValue != null && !oldValue.equals(value))
                    && target.getParameterInfo().isValid(value))
                newValue = (oldValue != null && value == null) ? "" : value;
            else newValue = null;
        }
        catch (RuntimeException e)
        {
            System.err.println("Error while validating: " + target + " -> "+ value);
            e.printStackTrace();
        }
    }

    public String getValue() {
        if (newValue!=null) return newValue;

        return Config.getInstance().getProperty(target);
    }

    public void apply() {
        if (newValue==null) return;

        Config.getInstance().setProperty(target, newValue);
        newValue = null;
    }

    public void revert() {
        newValue = null;
    }

    public boolean isRevertable() {
        return newValue != null;
    }

    public boolean isReadonly()
    {
        return target.getParameterInfo().getParameterType() == ParameterInfo.Type.READ_ONLY;
    }

    public void reset()
    {
        setValue(Config.getInstance().getDefaultValue(target));
    }

    public boolean isResetable()
    {
        return Config.getInstance().hasDefaultValue(target)
                && !Config.getInstance().isDefaultValue(target);
    }
}

class ParameterColumn extends TableColumn {

    private static final String[] columnNames = new String[] {
            Utilities.getString("PREF_TAB_ADVANCED_COL_PARAMETER"),
            Utilities.getString("PREF_TAB_ADVANCED_COL_VALUE") };
    protected static final int LABEL = 0;
    protected static final int VALUE = 1;

    protected ParameterColumn(int type, int width)
    {
        super(type, width);

        setCellRenderer(new TableCellRenderer() {

            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
                    boolean hasFocus, int row, int column)
            {
                Parameter p = (Parameter) value;
                switch (getModelIndex()) {
                    case LABEL: value = p.getLabel();
                                break;
                    case VALUE: value = p.getValue();
                                break;
                }
                Component c = table.getDefaultRenderer(p.getLabel().getClass())
                        .getTableCellRendererComponent(table, value, isSelected, hasFocus, row,
                                column);

                if (c instanceof JLabel) {
                    JLabel l = (JLabel) c;

                    boolean bold = false;
                    boolean italic =  false;
                    if (p.isResetable()) bold = true;
                    if (p.isRevertable()) italic = true;

                    l.setFont(l.getFont().deriveFont(
                            Font.PLAIN | (bold ? Font.BOLD : 0)
                                    | (italic && getModelIndex() == LABEL ? Font.ITALIC : 0)));

                    //provide tooltips when necessary
                    if (getWidth() < l.getPreferredSize().width)
                        l.setToolTipText(l.getText());
                    else l.setToolTipText(null);
                }

                return c;
            }

        });

        setHeaderValue(columnNames[getModelIndex()]);

        if (getModelIndex()==VALUE) setCellEditor(new PreferencesCellEditor());
    }
}

class PreferencesCellEditor extends AbstractCellEditor implements TableCellEditor, ActionListener,
        PropertyChangeListener {

    private Parameter p;

    private JComboBox fixedValueSelector = new JComboBox();
    private JTextField customEditor;
    private JPanel fileSelector;

    // there is a lot of trouble with custom cell editors, row heights and different look and feels
    // to ensure everything is working we alter the row height of the row that is currently edited 
    // especially a combobox working as cell editor under Nimbus is tricky
    // final solution: 4px higher rows + 1/2 of difference to preferred height added dynamicly
    private JTable currentTarget;
    private int currentRow;

    public PreferencesCellEditor()
    {
        fixedValueSelector.addActionListener(this);

        // ensures that we get the right borders around the Textfield for every laf
        // especially Nimbus makes trouble here
        JTable table = new JTable(new String[][] {{"1","2"}}, new String[] {"a","b"});
        customEditor = (JTextField) table.getDefaultEditor(String.class).getTableCellEditorComponent(table, "", true, 0, 0);
        customEditor.addActionListener(this);

        fileSelector = new JPanel(new BorderLayout());
        fileSelector.add(customEditor, BorderLayout.CENTER);
        JButton b = new JButton(" ... ");
        //avoid substance minimum button width
        b.setMinimumSize(new Dimension(b.getMinimumSize().height,b.getMinimumSize().height));
        b.setPreferredSize(b.getMinimumSize());
        b.setBorder(UIManager.getBorder("Table.cellNoFocusBorder"));
        b.addActionListener(this);
        b.setActionCommand("open-filechooser");
        fileSelector.add(b, BorderLayout.EAST);

        // react to laf changes
        UIManager.addPropertyChangeListener(this);
    }

    public void propertyChange(PropertyChangeEvent evt)
    {
        if (!evt.getPropertyName().equals("lookAndFeel")) return;

        SwingUtilities.updateComponentTreeUI(fixedValueSelector);
        SwingUtilities.updateComponentTreeUI(customEditor);
        SwingUtilities.updateComponentTreeUI(fileSelector);
    }

    public Object getCellEditorValue()
    {
        String value;
        switch (p.getKey().getParameterInfo().getParameterType()){
            case FIXED_VALUES_AND_CUSTOM:
            case FIXED_VALUES:
                value = (String) fixedValueSelector.getSelectedItem();
                break;
            case DIRECTORY:
            case FILE:
            default:
                value = customEditor.getText();
        }

        p.setValue(value);
        //reset row height
        if (currentTarget!=null)
            currentTarget.setRowHeight(currentRow, currentTarget.getRowHeight());
        return p;
    }

    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected,
            int row, int column)
    {
        p = (Parameter) value;
        value = p.getValue();

        currentRow = row;
        currentTarget = table;
        boolean editable = false;
        Component c;
        switch (p.getKey().getParameterInfo().getParameterType()) {
            case FIXED_VALUES_AND_CUSTOM:
                editable = true;
            case FIXED_VALUES:
                fixedValueSelector.setEditable(editable);
                fixedValueSelector.removeAllItems();
                for (String i : p.getKey().getParameterInfo().getFixedValues())
                    fixedValueSelector.addItem(i);
                fixedValueSelector.setSelectedItem(value);
                c = fixedValueSelector;
                break;
            case DIRECTORY:
            case FILE:
                customEditor.setText(p.getValue());
                //readd the custom editor
                //otherwise it won't show up if used alone in meanwhile
                fileSelector.add(customEditor, BorderLayout.CENTER);
                c = fileSelector;
                break;
            default:
                customEditor.setText(p.getValue());
                c = customEditor;
                break;
        }

        // resize current row a bit more for big cell editors
        table.setRowHeight(row, Math.max(table.getRowHeight(), table.getRowHeight()
                + (c.getPreferredSize().height - table.getRowHeight()) / 2));
        return c;
    }

    @Override
    public boolean isCellEditable(EventObject e)
    {
        if (e instanceof MouseEvent) 
            return ((MouseEvent)e).getClickCount() >= 2;
        return super.isCellEditable(e);
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getActionCommand().equals("open-filechooser")) 
        {
            //reuse customized Filechooser for player cmd
            if (p.getKey()==Config.Key.PLAYER_CMD) {
                File player = Player.showPlayerSelectionDialog(fileSelector);
                if (player != null) customEditor.setText(player.getAbsolutePath());
            }
            //use custom labels for destination dir selection
            else if (p.getKey()==Config.Key.DESTINATION_DIR) {
                File destDir = Utilities.showFileSelectionDialog(fileSelector, Utilities
                        .getString("SELDEST_TITLE"), Utilities.getString("SELDEST_APPROVE_LABEL"), Config
                        .getInstance().getProperty(Config.Key.DESTINATION_DIR), true);
                if (destDir != null) customEditor.setText(destDir.getAbsolutePath());
            }
            //reuse streamripper selection dialog
            else if (p.getKey()==Config.Key.STREAMRIPPER_CMD) {
                File ripper = StreamripperCommand.showSelectStreamripperDialog(fileSelector);
                if (ripper!=null) customEditor.setText(ripper.getAbsolutePath());
            }
        }
        stopCellEditing();
    }
}