/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.models;

import java.util.Observable;
import java.util.Observer;

import javax.swing.table.AbstractTableModel;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.core.storage.Favorites;


/**
 * Table Model for displaying and editing favorites
 * @author Andreas Kuehntopf
 */
public class FavoritesTableModel extends AbstractTableModel implements Observer 
{
    private Favorites _favs;

    /**
     * Constructs a new Favorites Table Model
     */   
    public 
    FavoritesTableModel()
    {        
        _favs = Favorites.getInstance();
        _favs.addObserver(this);   
    }
    
    /**
     * Returns if a given cell is editable
     * @param row
     * @param col
     * @return true if the cell is editable
     */
    @Override
    public boolean 
    isCellEditable(int row, int col)
    {
        // Everything is editable
        return true;
    }

    /**
     * Returns the number of rows
     * @return the number of rows
     */
    public int 
    getRowCount()
    {
        return _favs.getFavorites().size();
    }

    /**
     * Sets the value of a cell
     * @param value value to set
     * @param row row
     * @param col column
     */
    @Override
    public void 
    setValueAt(Object value, int row, int col)            
    {
        Favorites.getInstance().save();
        fireTableCellUpdated(row, col);
    }

    /**
     * Returns the number of columns
     * @return the number of columns
     */
    public int 
    getColumnCount()
    {
        return Station.Property.values().length;
    }

    /**
     * Returns the value of a specific cell
     * @param row row
     * @param col column
     * @return the value of the specified cell
     */
    public Object 
    getValueAt(int row, int col)
    {
        return _favs.getFavorites().get(row);
    }
    
    /**
     * This gets called whenever the FavoritesManager changes
     * its state
     * @param obs Observable
     * @param obj Message
     */
    public void 
    update(Observable obs, Object message)
    {
        fireTableDataChanged();
    }
}
