/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.views;

import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumnModel;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.gui.StreamtasticFrame;
import org.kuehntopf.streamtastic.gui.components.ColumnSelectionPopupMenu;
import org.kuehntopf.streamtastic.gui.components.StationColumn;
import org.kuehntopf.streamtastic.gui.components.StationPopupMenu;
import org.kuehntopf.streamtastic.gui.models.DownloadsTableModel;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * View for managing downloads
 * @author Andreas Kuehntopf
 */
public class DownloadsView extends ToolbarView
{    
    private JTable tblDownloads;
    private StreamtasticFrame sframe;

    /** 
     * Creates new form DownloadsView 
     */
    public DownloadsView(StreamtasticFrame sframe)
    {
        super(Utilities.getString("SIDEBARMENU_DOWNLOADS"), Utilities.loadIcon("large/radio.png"),
                Utilities.loadIcon("small/radio.png"));
        this.sframe = sframe;

        init();
        updateButtons();
    }    

    @Override
    protected JComponent initContentPane()
    {
        // Setup table
        tblDownloads = new JTable();
        tblDownloads.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        DownloadsTableModel model = new DownloadsTableModel();
        tblDownloads.setAutoCreateColumnsFromModel(false);
        TableColumnModel columnModel = new DefaultTableColumnModel();
        columnModel.addColumn(new StationColumn(Station.Property.RECORDING_STATE, 35, false));
        columnModel.addColumn(new StationColumn(Station.Property.NAME, 100, false));
        columnModel.addColumn(new StationColumn(Station.Property.CURRENT_TRACK, 150, false));
        columnModel.addColumn(new StationColumn(Station.Property.MIMETYPE, 25, false));
        columnModel.addColumn(new StationColumn(Station.Property.BITRATE, 25, false));
        columnModel.addColumn(new StationColumn(Station.Property.RECORDED_DATA_AMOUNT, 25, false));
        tblDownloads.getTableHeader().setComponentPopupMenu(new ColumnSelectionPopupMenu(columnModel));
        tblDownloads.setColumnModel(columnModel);
        tblDownloads.setModel(model);             
        tblDownloads.setRowSorter(StationColumn.getRowsorter(model));
        tblDownloads.getSelectionModel().addListSelectionListener(new ListSelectionListener() 
        {
            public void valueChanged(ListSelectionEvent arg0) 
            {
                updateButtons();
            }
        });

        // add popup menu
        JPopupMenu tblmenu = new StationPopupMenu();
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.PLAY));
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ADD_TO_FAVS));
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ABORT));
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ABORT_ALL));
        tblmenu.addSeparator();
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.COPY_URL_TO_CLIPBOARD));
        tblDownloads.setComponentPopupMenu(tblmenu);

        // configure action for double click
        tblDownloads.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e)
            {
                if (e.getClickCount()!=2) return;
                
                Action action = sframe.getSharedAction(StreamtasticFrame.StationAction.PLAY);
                
                if (action.isEnabled()) action.actionPerformed(
                        new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED,
                                StreamtasticFrame.StationAction.RECORD.toString(), e.getWhen(), e
                                        .getModifiers()));
            }
        });

        JScrollPane pane = new JScrollPane(tblDownloads);
        pane.setBorder(new CompoundBorder(new EmptyBorder(4,0,0,0), pane.getBorder()));
        return pane;
    }

    @Override
    protected void initToolbar(JToolBar toolbar)
    {
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.PLAY));
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ADD_TO_FAVS));
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ABORT));
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ABORT_ALL));
    }

    /**
     * Gets called when the View is activated
     */
    @Override
    protected void activate() 
    {
        updateButtons();
    }

    /**
     * Updates buttons according to 
     * selection
     */
    private void updateButtons()
    {
        sframe.updateStationActions(getSelectedStation());
    }
    
    /**
     * Gets the currently selected Ripper
     * @return current ripper or null
     */
    private Station getSelectedStation()
    {
        if (tblDownloads.getSelectedRow() < 0)
            return null;
        return (Station) tblDownloads.getValueAt(tblDownloads.getSelectedRow(), 0);
    }
}
