/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.views;

import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumnModel;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.core.storage.Favorites;
import org.kuehntopf.streamtastic.gui.StreamtasticFrame;
import org.kuehntopf.streamtastic.gui.components.FavoritesPanel;
import org.kuehntopf.streamtastic.gui.components.StationColumn;
import org.kuehntopf.streamtastic.gui.components.StationPopupMenu;
import org.kuehntopf.streamtastic.gui.models.FavoritesTableModel;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * View for managing favorite stations
 * @author akuehntopf
 */
public class FavoritesView extends ToolbarView
{    
    private FavoritesPanel favoritesPanel;
    private StreamtasticFrame sframe;

    public FavoritesView(StreamtasticFrame parent)
    {
        super(Utilities.getString("FAVS_HEADER_TEXT"), Utilities.loadIcon("large/favorites.png"),
                Utilities.loadIcon("small/favorites.png"));
        this.sframe = parent;

        init();
        updateButtons();
    }
    
    @Override
    protected JComponent initContentPane()
    {
        favoritesPanel = new FavoritesPanel();

        // Setup table
        FavoritesTableModel model = new FavoritesTableModel();
        TableColumnModel columnModel = new DefaultTableColumnModel();
        columnModel.addColumn(new StationColumn(Station.Property.NAME,Utilities.getString("FAVS_COL_DESC"),150,true));
        columnModel.addColumn(new StationColumn(Station.Property.URL,150));
        final JTable favTable = favoritesPanel.initTable(model, columnModel, StationColumn.getRowsorter(model));

        // init edit action
        Action cellEditAction = new AbstractAction("Edit", Utilities.loadIcon("small/edit.png")) {
            {
                favTable.getColumnModel().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e)
                    {
                        int col = favTable.getSelectedColumn();
                        col = col == -1 ? -1 : favTable.convertColumnIndexToModel(col);
                        if (col != -1)
                            switch (Station.Property.values()[col])
                            {
                                case NAME:
                                    putValue(Action.NAME, Utilities .getString("FAVS_COL_DESC_EDIT"));
                                    break;
                                case URL:
                                    putValue(Action.NAME, Utilities.getString("FAVS_COL_URL_EDIT"));
                                    break;
                            }
                    }
                });
            }
            public void actionPerformed(ActionEvent e)
            {
                int row = favTable.getSelectedRow();
                int column = favTable.getSelectedColumn();
                if (row==-1||column==-1) return;
                favTable.editCellAt(row, column);
                ((JComponent) favTable.getEditorComponent()).grabFocus();
            }
        };

        // configure action for double click
        favTable.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e)
            {
                if (e.getClickCount()!=2) return;
                
                Action action = sframe.getSharedAction(StreamtasticFrame.StationAction.valueOf(Config
                        .getInstance().getProperty(Config.Key.FAVS_STATION_DOUBLE_CLICK_ACTION)));
                
                if (action!= null && action.isEnabled()) action.actionPerformed(
                        new ActionEvent(e.getSource(), ActionEvent.ACTION_PERFORMED,
                                StreamtasticFrame.StationAction.RECORD.toString(), e.getWhen(), e
                                        .getModifiers()));
            }
        });

        // add popup menu
        JPopupMenu tblmenu = new StationPopupMenu();
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.PLAY));
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.RECORD));
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ABORT));
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.REM_FROM_FAVS));
        tblmenu.addSeparator();
        tblmenu.add(sframe.getSharedAction(StreamtasticFrame.StationAction.COPY_URL_TO_CLIPBOARD));
        tblmenu.add(cellEditAction);
        favTable.setComponentPopupMenu(tblmenu);

        favTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() 
        {            
            public void valueChanged(ListSelectionEvent arg0) 
            {
                updateButtons();
            }
        });

        Action quickAddAction = new AbstractAction() {
            public void actionPerformed(ActionEvent evt)
            {
                JTextField tf = (JTextField) evt.getSource();
                String url = tf.getText();
                try
                {
                    new URL(url);
                    Favorites.getInstance().addFavorite(url, "", null, null);
                    tf.setText("");
                    favTable.editCellAt(favTable.convertRowIndexToView(favTable.getRowCount() - 1),
                            favTable.convertColumnIndexToView(Station.Property.NAME.ordinal()));

                    JComponent editor = (JComponent) favTable.getEditorComponent();
                    favTable.scrollRectToVisible(editor.getBounds());
                    editor.grabFocus();
                }
                catch (MalformedURLException ex)
                {
                    Utilities.showError(Utilities.getString("FAVS_INVALID_URL"));
                }
            }
        };
        favoritesPanel.setQuickAddAction(quickAddAction);

        return favoritesPanel;
    }

    @Override
    protected void initToolbar(JToolBar toolbar)
    {
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.PLAY));
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.RECORD));
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.ABORT));
        toolbar.add(sframe.getSharedAction(StreamtasticFrame.StationAction.REM_FROM_FAVS));
    }

    /**
     * Gets called when the View is activated
     */
    @Override
    protected void activate() 
    {
        updateButtons();
    }

    private void updateButtons()
    {
        sframe.updateStationActions(favoritesPanel.getSelectedFavorite());
    }
}
