/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008, 2009 Andreas Kuehntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.utils;

import java.util.List;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public abstract class SelectionMaintainer<T> {

    public abstract void fireAdded(int start, int end);
    public abstract void fireRemoved(int start, int end);
    public abstract void fireReplaced(int start, int end);

    public void updateList(List<T> target, List<T> source, int selected)
    {
        // update without altering current selection
        // works also in case that newIdx==-1
        int newIdx = selected == -1 ? -1 : source.indexOf(target.get(selected));

        if (newIdx == -1 && selected != -1)
            // the selected item is no longer present in source list but was in target list
            // so current selected item must be removed to clear selection
            remove(selected, selected + 1, target);
        else
        {
            // adjust position of selected item in target list to reflect new position in source list
            // nothing will be done if a==b==-1
            int a = Math.min(newIdx, selected);
            int b = a == -1 ? -1 : Math.max(newIdx, selected);

            replace(0, a, target, source, 0);

            if (newIdx - selected > 0)
                add(a, b, target, source, a);
            else
                remove(a, b, target);
        }

        newIdx = newIdx == -1 ? 0 : newIdx;
        replace(newIdx, Math.min(target.size(), source.size()), target, source, newIdx);

        if (target.size() > source.size())
            remove(source.size(), target.size(), target);
        else
            add(target.size(), source.size(), target, source, target.size());
    }

    private void replace(int start, int end, List<T> target, List<T> source, int offset)
    {
        if (start >= end)
            return;
        for (int i = start; i < end; i++)
            target.set(i, source.get(i - start + offset));
        fireReplaced(start, end - 1);
    }

    private void add(int start, int end, List<T> target, List<T> source, int offset)
    {
        if (start >= end)
            return;
        for (int i = start; i < end; i++)
            target.add(i, source.get(i - start + offset));
        fireAdded(start, end - 1);
    }

    private void remove(int start, int end, List<T> target)
    {
        if (start >= end)
            return;
        for (int i = start; i < end; i++)
            target.remove(start);
        fireRemoved(start, end - 1);
    }
}
