/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.utils;

import java.awt.Component;
import java.io.File;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * Utilities for i18n, etc
 * @author akuehntopf
 */
public class Utilities 
{   
    public static String bundle = 
            "org.kuehntopf.streamtastic.resources.i18n.streamtastic";
        
    /**
     * Loads an icon at the specific path
     * relative to the icons subdirectory
     * @param path Relative path
     * @return Loaded ImageIcon
     */
    public static ImageIcon 
    loadIcon(String path)
    {
        String pathPrefix = "/org/kuehntopf/streamtastic/resources/icons";     
        return new ImageIcon(Utilities.class.getResource(
                pathPrefix + "/" + path));
    }

    /**
     * Returns the User's home directory
     * @return Home dir
     */
    public static String 
    getHome()
    {
       return System.getProperty("user.home");
    }

    /**
     * Gets a localized String
     * @param key key to search for
     */
    public static String
    getString(String key)
    {
        try
        {
            return ResourceBundle.getBundle(bundle).getString(key);
        }
        catch (MissingResourceException e)
        {
            System.err.println("Missing String for key \"" + key + "\" and locale "
                    + ResourceBundle.getBundle(bundle).getLocale() + ".");
            return key;
        }
    }
    
    /**
     * Returns the User's storage directory
     * @return the storage directory
     */
    public static String 
    getStorageDirectory()
    {
        String dir = getHome() + File.separator + ".streamtastic";
        File f = new File(dir);
        if (!f.exists())
        {
            f.mkdir();
        }
        return dir;
    }
    
    /**
     * Displays an Error message
     * @param text text to display 
     */
    public static void 
    showError(String text)
    {            
        JOptionPane.showMessageDialog(null, text, getString("ERROR"), 
                JOptionPane.ERROR_MESSAGE);
    }

    public static File
    showFileSelectionDialog(Component parent, String title, String approveLabel,
            String location, boolean directoriesOnly)
    {
        if (location==null) location = Utilities.getHome();
        
        JFileChooser filechooser = new JFileChooser(location);
        filechooser.setDialogTitle(title);
        if (directoriesOnly) filechooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        if (JFileChooser.APPROVE_OPTION == filechooser.showDialog(parent,
                approveLabel))
            return filechooser.getSelectedFile();
        
        return null;
    }
}
